package com.sample;

import java.sql.SQLException;
import java.util.List;

import javax.naming.InitialContext;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.transaction.UserTransaction;

import org.apache.log4j.Logger;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.ProcessInstance;
import org.h2.tools.DeleteDbFiles;
import org.h2.tools.Server;
import org.jbpm.task.Group;
import org.jbpm.task.TaskService;
import org.jbpm.task.User;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.TaskServiceSession;

import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is an example that shows JBPM5 API usage using Local task service.
 */
public class ProcessLocalTaskTaskAPITest {

	static Logger logger = Logger.getLogger(ProcessLocalTaskTaskAPITest.class);
	
	public static final void main(String[] args) {
		
		/*
		 * Start local h2 datbase
		 * This is not required if the application connects to a remote database
		 */
	
		
		try {
			DeleteDbFiles.execute("", "JPADroolsFlow", true);
			 Server h2Server = Server.createTcpServer(new String[0]);
			h2Server.start();
		} catch (SQLException e) {
			throw new RuntimeException("can't start h2 server db",e);
		}
	  
		
		try {
			
			UserTransaction ut = (UserTransaction) new InitialContext().lookup( "java:comp/UserTransaction" );
            //ut.begin();

			PoolingDataSource ds1 = new PoolingDataSource();
	       
			
			//sample H2 database configuration
			
			
			ds1.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
	    	ds1.setUniqueName("jdbc/jbpm-ds");
	    	ds1.setMaxPoolSize(5);
	    	ds1.setAllowLocalTransactions(true);
	    	ds1.getDriverProperties().setProperty("driverClassName", "org.h2.Driver");
	    	ds1.getDriverProperties().setProperty("url", "jdbc:h2:tcp://localhost/JPADroolsFlow");
	    	ds1.getDriverProperties().setProperty("user", "sa");
	    	ds1.getDriverProperties().setProperty("password", "");
	    	
			
	    	
			//Sample Oracle datasource configuration
	    	/*
			ds1.setUniqueName( "jdbc/jbpm-ds" );
	        ds1.setClassName( "oracle.jdbc.xa.client.OracleXADataSource" );
	        ds1.setMaxPoolSize( 5 );
	        ds1.setAllowLocalTransactions( true );
	        
	        ds1.getDriverProperties().put( "user","user" );
	        ds1.getDriverProperties().put( "password","pw" );
	        ds1.getDriverProperties().put( "URL","jdbc:oracle:thin:@oraclehost:1521:jbpmdb" );
			*/
			//Sample Mysql datasource configuration
	    	/*
			   ds1 = new PoolingDataSource();
		       ds1.setUniqueName( "jdbc/jbpm-ds" );
		       ds1.setClassName( "com.mysql.jdbc.jdbc2.optional.MysqlXADataSource" );
		        
		       ds1.setMaxPoolSize( 3 );
		       ds1.setAllowLocalTransactions( true );
		       ds1.getDriverProperties().put( "user","user" );
		       ds1.getDriverProperties().put( "password","user" );
		       ds1.getDriverProperties().put( "URL",
		                                     "jdbc:mysql://mysqlserver:3306/jbpm5db" );
             */
		        
	        ds1.init();
	        //ut.commit();
	        
	        ut.begin();
	       
	        System.setProperty("jbpm.usergroup.callback", "org.jbpm.task.service.DefaultUserGroupCallbackImpl");
	        
	        /*
	         * * Create EntityManagerFactory and register it in the environment
	         */
	        EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa");
	        
	        /*
	    	 * Create the knowledge session that uses JPA to persists runtime state
	    	 */
	        StatefulKnowledgeSession ksession = JbpmAPIUtil.createKnowledgeSession("HelloProcess.bpmn",emf);
	        org.jbpm.task.service.TaskService tservice = JbpmAPIUtil.getService(emf); 
	        
	        /*
	         * Get the local task service
	         */
	        TaskService taskService = JbpmAPIUtil.getTaskService(ksession,tservice,emf);
			
			ProcessInstance processInstance = ksession.startProcess("Hello");
			
			/*
			 * Retrive the tasks owned by a user
			 */
			
			List<TaskSummary> list = taskService.getTasksAssignedAsPotentialOwner("krisv", "en-UK");
			TaskSummary task = list.get(0);
			//logger.debug("Kris is executing task " + task.getName());
			System.out.println("krisv is executing task " + task.getName());
			taskService.start(task.getId(), "krisv");
			taskService.complete(task.getId(), "krisv", null);
			//logger.debug("completed the human task " );
			
			 
			System.out.println("completed the human task " );
			ut.commit();
		} catch (Throwable t) {
			t.printStackTrace();
			//logger.error(t.getMessage(), t.getCause());
			
		}
		

	
	
	
		}
	
    
}
